#
# Exercice 1
#

movie<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/movie_buzz.csv',
                header=TRUE,sep=",")

# Preprocessing

# declaring the factors
movie$MPRATING<-as.factor(movie$MPRATING)
levels(movie$MPRATING)<-c("G","PG","PG13","R")
movie$SEQUEL<-as.factor(movie$SEQUEL)
movie$ACTION<-as.factor(movie$ACTION)
movie$COMEDY<-as.factor(movie$COMEDY)
movie$ANIMATED<-as.factor(movie$ANIMATED)
movie$HORROR<-as.factor(movie$HORROR)
# Log transformation
movie$BOX<-log(movie$BOX)
movie$BUDGET<-log(movie$BUDGET)

X<-model.matrix(BOX~.,data=movie)

# Q1
n<-nrow(movie)
ntrain<-40
ntest<-n-ntrain
train<-sample(n,ntrain)
movie.train<-movie[train,]
movie.test<-movie[-train,]
X.train<-X[train,]
X.test<-X[-train,]

# Q2
library("leaps")
# Forward selection 
reg.forward<-regsubsets(BOX~.,data=movie.train,method='forward',
                        nvmax=15)
plot(reg.forward,scale="bic")
plot(reg.forward,scale="adjr2")
res<-summary(reg.forward)

best.bic<-which.min(res$bic)
coef(reg.forward,best.bic)

ypred<-X.test[,res$which[best.bic,]]%*%coef(reg.forward,best.bic)
mse_forward_bic<-mean((ypred-movie.test$BOX)^2)
plot(movie.test$BOX,ypred)
abline(0,1)

best.adjr2<-which.max(res$adjr2)
coef(reg.forward,best.adjr2)

ypred<-X.test[,res$which[best.adjr2,]]%*%coef(reg.forward,best.adjr2)
mse_forward_adjr2<-mean((ypred-movie.test$BOX)^2)

print(c(mse_forward_bic,mse_forward_adjr2))

# Backward selection 
reg.backward<-regsubsets(BOX~.,data=movie.train,method='backward')
plot(reg.backward,scale="bic") # same as forward
plot(reg.backward,scale="adjr2") # this one is different

res<-summary(reg.backward)

best.adjr2<-which.max(res$adjr2)
coef(reg.backward,best.adjr2)

ypred<-X.test[,res$which[best.adjr2,]]%*%coef(reg.backward,best.adjr2)
mse_backward_adjr2<-mean((ypred-movie.test$BOX)^2)

print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2))


# Optimal subset

reg.exhaustive<-regsubsets(BOX~.,data=movie.train,method='exhaustive',
                           nvmax=15)
plot(reg.exhaustive,scale="bic") # same as before
plot(reg.exhaustive,scale="adjr2")


# Ridge
library(glmnet)

cv.out<-cv.glmnet(X.train[,-1],movie.train$BOX,alpha=0,standardize=TRUE)
plot(cv.out)
fit1<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.min,
             alpha=0,standardize=TRUE)
fit2<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.1se,
             alpha=0,standardize=TRUE)
# Minimum CV error model
ridge.pred1<-predict(fit1,newx=X.test[,-1])
mse_ridge1<-mean((movie.test$BOX-ridge.pred1)^2)
# One SE rule
ridge.pred2<-predict(fit2,newx=X.test[,-1])
mse_ridge2<-mean((movie.test$BOX-ridge.pred2)^2)
print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2,
        mse_ridge1,mse_ridge2))

# lasso
cv.out<-cv.glmnet(X.train[,-1],movie.train$BOX,alpha=1,standardize=TRUE)
plot(cv.out)
fit1<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.min,
             alpha=1,standardize=TRUE)
fit2<-glmnet(X.train[,-1],movie.train$BOX,lambda=cv.out$lambda.1se,
             alpha=1,standardize=TRUE)

lasso.pred1<-predict(fit1,newx=X.test[,-1])
mse_lasso1<-mean((movie.test$BOX-lasso.pred1)^2)
lasso.pred2<-predict(fit2,newx=X.test[,-1])
mse_lasso2<-mean((movie.test$BOX-lasso.pred2)^2)

print(c(mse_forward_bic,mse_forward_adjr2,mse_backward_adjr2,
        mse_ridge1,mse_ridge2, mse_lasso1,mse_lasso2))

#
# Exercice 2
#
credit<-read.csv('/Users/Thierry/Documents/R/Data/Economics/default_credit_card.csv',
                 sep=";",header=TRUE)
credit$X2<-as.factor(credit$X2)
levels(credit$X2)<-c("M","F")
credit$X3<-as.factor(credit$X3)
credit$X4<-as.factor(credit$X3)
credit$Y<-as.factor(credit$Y)

# Q1
n<-nrow(credit)
ntrain<-20000
ntest<-n-ntrain
train<-sample(n,ntrain)
credit.train<-credit[train,]
credit.test<-credit[-train,]

# Q2
library(MASS)
library(naivebayes)

# 10-fold cross-validation
K<-10
folds=sample(1:K,ntrain,replace=TRUE)
CV<-matrix(0,K,4)

for(k in (1:K)){
  print(k)
  fit.lda<- lda(Y~.,data=credit.train[folds!=k,-c(2,3,4)])
  pred.lda<-predict(fit.lda,newdata=credit.train[folds==k,])
  CV[k,1]<-sum(pred.lda$class!=credit.train$Y[folds==k])
  fit.qda<- qda(Y~.,data=credit.train[folds!=k,-c(2,3,4)])
  pred.qda<-predict(fit.qda,newdata=credit.train[folds==k,])
  CV[k,2]<-sum(pred.qda$class!=credit.train$Y[folds==k])
  fit.nb<- naive_bayes(Y~.,data=credit.train[folds!=k,])
  pred.nb<-predict(fit.nb,newdata=credit.train[folds==k,],type="class")
  CV[k,3]<-sum(pred.nb!=credit.train$Y[folds==k])
  fit.logreg<- glm(Y~.,data=credit.train[folds!=k,],family=binomial)
  pred.logreg<-predict(fit.logreg,newdata=credit.train[folds==k,],type='response')
  CV[k,4]<-sum(as.factor(as.numeric(pred.logreg>0.5))!=credit.train$Y[folds==k])
}
err_cv=colSums(CV)/ntrain
print(err_cv)

# Q3
# test error rate of the best model
fit.lda<- lda(Y~.,data=credit.train[,-c(2,3,4)])
pred.lda<-predict(fit.lda,newdata=credit.test)
err.lda<-mean(pred.lda$class!=credit.test$Y)

# We fit LDA on the whole dataset
fit.lda<- lda(Y~.,data=credit[,-c(2,3,4)])

# Q4
library(klaR)
fit.rda<-rda(Y~.,data=credit.train[,-c(2,3,4)])
pred.rda<-predict(fit.rda,newdata=credit.test)
fit.rda$regularization
err.rda<-mean(pred.rda$class!=credit.test$Y)

#
# Exercise 3
#

# Q1
data("USArrests")
pca<-princomp(USArrests,cor=TRUE)
Z<-pca$scores
lambda<-pca$sdev^2

# Q2
biplot(pca)

# Q3
plot(cumsum(lambda)/sum(lambda),type="b",
     xlab="q",ylab="proportion of explained variance")




#
# Exercise 4 
#

movie<-read.csv('/Users/Thierry/Documents/R/Data/Economics/Greene/movie_buzz.csv',
                header=TRUE,sep=",")

# Preprocessing

# declaring the factors
movie$MPRATING<-as.factor(movie$MPRATING)
levels(movie$MPRATING)<-c("G","PG","PG13","R")
movie$SEQUEL<-as.factor(movie$SEQUEL)
movie$ACTION<-as.factor(movie$ACTION)
movie$COMEDY<-as.factor(movie$COMEDY)
movie$ANIMATED<-as.factor(movie$ANIMATED)
movie$HORROR<-as.factor(movie$HORROR)
# Log transformation
movie$BOX<-log(movie$BOX)
movie$BUDGET<-log(movie$BUDGET)

pca<-princomp(movie[,10:13],cor=TRUE)
Z<-pca$scores
lambda<-pca$sdev^2

plot(cumsum(lambda)/sum(lambda),type="b",
     xlab="q",ylab="proportion of explained variance")

movie$BUZZ<-Z[,1]
movie<-movie[,-(10:13)]
X<-model.matrix(BOX~.,data=movie)

n<-nrow(movie)
ntrain<-40
ntest<-n-ntrain
train<-sample(n,ntrain)
movie.train<-movie[train,]
movie.test<-movie[-train,]
X.train<-X[train,]
X.test<-X[-train,]

library("leaps")
# Forward selection + BIC
reg.forward<-regsubsets(BOX~.,data=movie.train,method='forward',
                        nvmax=15)
plot(reg.forward,scale="bic")
res<-summary(reg.forward)

best.bic<-which.min(res$bic)
coef(reg.forward,best.bic)

ypred<-X.test[,res$which[best.bic,]]%*%coef(reg.forward,best.bic)
mse_forward_bic<-mean((ypred-movie.test$BOX)^2)
plot(movie.test$BOX,ypred)
abline(0,1)

# Lasso
library(glmnet)
cv.out<-cv.glmnet(X.train,movie.train$BOX,alpha=1,standardize=TRUE)
plot(cv.out)
fit<-glmnet(X.train,movie.train$BOX,lambda=cv.out$lambda.min,
             alpha=1,standardize=TRUE)
lasso.pred<-predict(fit,newx=X.test)
mse_lasso<-mean((movie.test$BOX-lasso.pred)^2)

print(c(mse_forward_bic,mse_lasso))
